#!/usr/bin/env bash

# This file is part of I38.
                                                                                                                                                                          
# I38 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
                                                                                                                                                                          
# I38 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
                                                                                                                                                                          
# You should have received a copy of the GNU General Public License along with I38. If not, see <https://www.gnu.org/licenses/>.
                                                                                                                                                                          
 
#check for acpi
if command -v acpi &> /dev/null; then
    bat="$(acpi -b)"
    spd-say -P important -Cw "$bat"
else
    find /sys/class/power_supply -type l -exec bash -c '
        for i ; do
            if [[ -e "$i/capacity" ]]; then
                bat="${i##*/}"
                bat="${bat//BAT/Battery }"
                bat="${bat}: $( { cat "${i}/status";echo -n ", "; cat "${i}/capacity"; } | tr -d \\n) percent."
                spd-say -P important -Cw "$bat"
            fi
        done
    ' _ {} \;
fi
