;;; Directory Local Variables            -*- no-byte-compile: t -*-
;;; For more information see (info "(emacs) Directory Variables")

((nil . ((vc-prepare-patches-separately . nil)
         (vc-default-patch-addressee . "billy@wolfe.casa"))))
